//+-------------------------------------------------------------------
//
//  
//  Copyright (C) Silicon Software GmbH, 1997-98.
//
//  File:       menable.h
//
//  Contents:   
//              
//
//
//--------------------------------------------------------------------

#ifndef __MENABLEH
#define __MENABLEH

#include <stdio.h>
#include "os_type.h"
#include "mem_siso.h"
#include "hap.h"

//#define ME_FUNC_PTR * 

typedef void microenable;
typedef void fpga_design;
typedef unsigned long eeprom;
typedef unsigned long readback_array;
typedef unsigned long xc_array;
typedef unsigned long short_info;
typedef unsigned long Idcode_struct;
typedef unsigned long EeepromBoardInfo;

typedef struct service_s {
	unsigned long version;
	unsigned long driver_version;
	int	build;
	unsigned long interrupt_count;
	long timer0;
	long timer1;
	long timer2;
	long timer3;
	unsigned long TimeCount;
	long timeout0;
	long timeout1;
	int  EventWaitSingleLoopTimeout0; 
	int  EventWaitSingleLoopTimeout1; 
	int  EventStopSingleLoopTimeout0; 
	int  EventStopSingleLoopTimeout1; 
	int  EventWCntStart0,EventWCntStop0;
	int  EventWCntStart1,EventWCntStop1;
	int   act_pic_cnt[2];
	int	 err_cnt;
	char err_msg[1024];
} service;
//#include "fgrab.h"

#ifdef __cplusplus
extern "C" {
#endif


//#define COMPILE

#ifdef COMPILE
#define dlldir __declspec( dllexport)
#else
#define dlldir __declspec( dllimport)
#endif

// 
// Prototypes
//

//
// init and free
//

microenable *initialize_microenable();
microenable *initialize_multienable(int nr);
microenable *reinitialize_microenable();
microenable *reinitialize_multienable(int nr);
int free_microenable(microenable *board);
int init_microlib(char*);
microenable* init_manual(int type,int nFlag);

//
// Mapping
//

extern int (ME_FUNC_PTR map_runtime_base)(microenable *board);
extern int (ME_FUNC_PTR map_rombase)(microenable *board);
extern int (ME_FUNC_PTR unmap_runtime_base)(microenable *board);
extern int (ME_FUNC_PTR unmap_rombase)(microenable *board);

extern int (ME_FUNC_PTR map_plx_base)(microenable *board);
extern int (ME_FUNC_PTR map_fpga_base)(microenable *board);
extern int (ME_FUNC_PTR map_slave_base)(microenable *board);
extern int (ME_FUNC_PTR eeprom_readout)(microenable *board);
extern int (ME_FUNC_PTR eeprom_readout_extern)(microenable *board,eeprom *);
extern int (ME_FUNC_PTR load_my_eeprom)(microenable *board, eeprom *eeprom_values);
extern int (ME_FUNC_PTR init_my_eeprom)(microenable *board,eeprom *eeprom_values,EeepromBoardInfo *info);

extern int (ME_FUNC_PTR WriteDataToDriver)(microenable *board,unsigned long data,int address);

//extern int (*reset)(microenable *board);
extern int (ME_FUNC_PTR init_abort)(microenable *board);
extern int (ME_FUNC_PTR init_expansion_rom)(microenable *board);
extern int (ME_FUNC_PTR init_slave_0)(microenable *board);
extern int (ME_FUNC_PTR init_clk_system)(microenable *board);

//
// Design prototypes
//

extern fpga_design *(ME_FUNC_PTR load_design)(microenable*, char *bin_file_name);
extern int (ME_FUNC_PTR unload_design)(fpga_design *design);
extern int (ME_FUNC_PTR prepare_design)(fpga_design *design);
extern int (ME_FUNC_PTR configure_fpga)(microenable *board, fpga_design *design);
extern int (ME_FUNC_PTR readback_fpga)(microenable *board, fpga_design *design);

//extern int (ME_FUNC_PTR read_pci_config_registers_small)(microenable *board,PPCI_COMMON_CONFIG );

//
// Clock Functions
//

// ------------------------------------------------------------------------------ //
//   begin clock system
// ------------------------------------------------------------------------------ //
enum ClkSource  {DESIGN_CLK2, DESIGN_CLK, LOCAL_BUS_CLK, CLOCK_DISABLED};
enum ResetState {RESET_OFF, RESET_ON};
enum ClkState   {UNLOCKED,  LOCKED};

extern int (ME_FUNC_PTR ClkSelectRam0)(microenable *board,int nId);
extern int (ME_FUNC_PTR ClkSelectRam1)(microenable *board,int nId);

extern int (ME_FUNC_PTR load_my_eeprom)(microenable *board, eeprom *eeprom_values);

//extern int (*reset)(microenable *board);
extern int (ME_FUNC_PTR init_abort)(microenable *board);
extern int (ME_FUNC_PTR init_expansion_rom)(microenable *board);
extern int (ME_FUNC_PTR init_slave_0)(microenable *board);
extern int (ME_FUNC_PTR init_clk_system)(microenable *board);

extern int (ME_FUNC_PTR set_cmc_clk)(microenable *board, fpga_design *design,int sel);
extern int (ME_FUNC_PTR set_fpga_clk)(microenable *board, fpga_design *design,int freq);
extern int (ME_FUNC_PTR set_fpga_clk_slow)(microenable *board, fpga_design *design,int freq);
extern int (ME_FUNC_PTR set_plx_clk)(microenable *board, fpga_design *design,int freq);

extern int (ME_FUNC_PTR get_plx_clk)(microenable *board, fpga_design *design);
extern int (ME_FUNC_PTR get_fpga_clk)(microenable *board, fpga_design *design);

extern int (ME_FUNC_PTR set_slink_source)(microenable *board);
extern int (ME_FUNC_PTR set_slink_destination)(microenable *board);

//
// RBT and Hap Files
//

extern HAP *(ME_FUNC_PTR read_hap)(microenable*,char *file_name);
extern unsigned long *(ME_FUNC_PTR GetAccessPointer)(microenable *board);
extern int (ME_FUNC_PTR FreeAccessPointer)(microenable *board);
extern unsigned long *(ME_FUNC_PTR GetAccessPointerNC)(microenable *board);
extern unsigned long *(ME_FUNC_PTR GetRuntimeBase)(microenable *board);
extern unsigned long *(ME_FUNC_PTR GetRomBase)(microenable *board);
extern unsigned long *(ME_FUNC_PTR GetServiceBase)(microenable *pBoard);
extern int (ME_FUNC_PTR LoadRbt)(char *name,rbt_info **ri);
extern int (ME_FUNC_PTR LoadRbtVirtex)(char *name,rbt_info **ri);
extern int (ME_FUNC_PTR LoadBit)(char *name,rbt_info **ri);
extern int (ME_FUNC_PTR FreeRbt)(rbt_info *ri);
extern int (ME_FUNC_PTR WriteHapFileByName)(char *name,rbt_info *ri);

extern HapTagMaster* (ME_FUNC_PTR ReadDesignInfoFile)	(char *name);

extern int (ME_FUNC_PTR CreateMultiHap)(char *name);
extern int (ME_FUNC_PTR ScanHapfile)(char *filename,hapfile_index *index);
extern int (ME_FUNC_PTR AddHapData)(char *cpHapName,int nTag, HapData *data);
extern HAP (ME_FUNC_PTR ReadMultiHap)(microenable *board,char *file_name);
extern char *(ME_FUNC_PTR Tag2String)(unsigned long tag);

extern int (ME_FUNC_PTR TestAndLockDesign)	(microenable *board);
extern int (ME_FUNC_PTR FreeDesignLock)		(microenable *board);

//
// Dma Functions
//

extern int (ME_FUNC_PTR WriteDmaData)(unsigned long* ptr,long len,microenable *board);
extern int (ME_FUNC_PTR ReadDmaData)(unsigned long* ptr,long len,microenable *board);
extern int (ME_FUNC_PTR WriteDmaDataExt)(unsigned long* ptr,long len,int chan,long offset,int inc,microenable *board);
extern int (ME_FUNC_PTR ReadDmaDataExt)(unsigned long* ptr,long len,int chan,long offset,int inc,microenable *board);
extern int (ME_FUNC_PTR WriteDmaDataExtCut)(unsigned long* ptr,long len,int chan,long offset,int inc,int maxline,int row_len,int eff_row_len,microenable *board);
extern int (ME_FUNC_PTR DmaJpegExt)(	unsigned long* ptr_write,long write_len,unsigned long* ptr_read,long read_len,int maxline,int row_len,int eff_row_len,microenable *board);
extern int (ME_FUNC_PTR WriteDmaDataExtCutline)(unsigned long* ptr,long len,int chan,long offset,int inc,int maxline,int row_len,int eff_row_len,microenable *board);
extern int (ME_FUNC_PTR WriteDmaDemandData)(unsigned long* ptr,long len,int chan,long offset,int inc,microenable *board);
extern int (ME_FUNC_PTR ReadDmaDemandData)(unsigned long* ptr,long len,int chan,long offset,int inc,microenable *board);
extern int (ME_FUNC_PTR DmaJpeg)(unsigned long* ptr_write,long write_len, unsigned long* read_write,long read_len,microenable *board);

extern int (ME_FUNC_PTR DmaVirtReadExtAsync)(microenable*,int,int,int,dma_mem*,int,int,int,int);
extern int (ME_FUNC_PTR DmaVirtWriteExtAsync)(microenable*,int,int,int,dma_mem*,int,int,int,int);
extern int (ME_FUNC_PTR DmaVirtWaitForBuffer)(microenable *board,int chan,int subbufnum,dma_mem *mem,int timeout);
extern int (ME_FUNC_PTR DmaVirtWaitForBufferEx)(microenable *board,int chan,int subbufnum,dma_mem *mem,int timeout,HANDLE Event);
extern int (ME_FUNC_PTR DmaVirtWaitForBufferHandshake)(microenable *board,int nPicNr,int nFlag,dma_mem *pMem,int nTimeout);

extern int (ME_FUNC_PTR DmaVirtTestSubbufRdy)(microenable *board,int chan,int subbufnum,dma_mem *mem);
extern int (ME_FUNC_PTR DmaVirtTestBufStatus)(microenable *board,int chan,dma_mem* mem);
extern int (ME_FUNC_PTR DmaVirtPulseNextBuffer)(microenable *board,int chan,dma_mem* mem);
extern int (ME_FUNC_PTR DmaVirtStopTransfer)(microenable *board,int nChan,int nMode);
extern int (ME_FUNC_PTR DmaVirtStopTransferEx)(microenable *board,int nChan,int nMode,dma_mem *pMem);
extern int (ME_FUNC_PTR DmaVirtStartRingbuf)(microenable *pBoard,int nChan,int nDmaLength,int nOffset,dma_mem *pMem);
extern int (ME_FUNC_PTR DmaVirtStopRingbuf) (microenable *pBoard,int nChan);

extern int (ME_FUNC_PTR DmaReadWriteExt)	
(unsigned long* ptr_write,long write_len,long write_offset,unsigned long* ptr_read,
 long read_len,long read_offset,
 int flags,microenable *pBoard);

extern int	(ME_FUNC_PTR DmaVirtGetBufferStatus)	(microenable *board,dma_mem *dm,int nParam,int nData,int *Outdata);

//
// Erweiterte DMA Funktionen
//

extern unsigned long	(ME_FUNC_PTR GetDmaLength)			(microenable *board,dma_mem *pMem, int nSubbufNr);
extern unsigned long	(ME_FUNC_PTR GetDmaTag)				(microenable *board,dma_mem *pMem, int nSubbufNr);
extern unsigned int		(ME_FUNC_PTR GetDmaTimeStamp)		(microenable *board,dma_mem *pMem,int nSubbufNr);
extern unsigned __int64	(ME_FUNC_PTR GetDmaTimeStampLong)	(microenable *board,dma_mem *pMem,int nSubbufNr);

//
// Error Handling 
//

extern void (ME_FUNC_PTR Me_ClearErrMsg)(microenable *board);
extern void (ME_FUNC_PTR Me_SetErrMsg)(microenable *board,char *);
extern char *(ME_FUNC_PTR Me_GetErrMsg)(microenable *board);
extern int  (ME_FUNC_PTR Me_GetErrCode)(microenable *board);
extern int  (ME_FUNC_PTR Me_GetErrMsgExt)(microenable *board,int *xerr_code,int *xerr_code_int,int *xerr_line_nr,unsigned long *xerr_data,char *xerr_filename,int *xerr_cnt,int flag);

//
// Interrupt
//

extern int (ME_FUNC_PTR SleepUntilInterrupt)(microenable *board);
extern int (ME_FUNC_PTR SetTimeout)(microenable *board,int nr);

// Das Locken von Speicher , Testfuctions

extern int (ME_FUNC_PTR MMLock)(unsigned long* ptr_write,	long write_len, unsigned long* ptr_read,	long read_len,microenable *board);
extern int (ME_FUNC_PTR MMUnlock)(microenable *board);

extern dma_mem*	(ME_FUNC_PTR MMSetVirtualBuffer)(microenable *board, int size,int nr);
extern int		(ME_FUNC_PTR MMFreeVirtualBuffer)(microenable *board,dma_mem*);
extern dma_mem*	(ME_FUNC_PTR MMSetVirtualUserBuffer)(microenable *board, int size,int nr);
extern int		(ME_FUNC_PTR MMFreeVirtualUserBuffer)(microenable *board,dma_mem*);
extern dma_mem* (ME_FUNC_PTR MMSetPhysicalLineBuffer)( microenable *board, int size,int nr,unsigned long,int,int);
extern void*	(ME_FUNC_PTR MMAddVirtualUserBuffer)	(microenable* board,dma_mem* dma_buf,unsigned long *buffer,int length,int subbufNr);
extern int		(ME_FUNC_PTR MMDelVirtualUserBuffer)	(microenable* board,dma_mem* dma_buf,void *bufferHandle);


// -----------------------------------------------------------
// Auslsen eines lokalen Interrupts

extern int (ME_FUNC_PTR activate_linto)(microenable *board);
extern int (ME_FUNC_PTR deactivate_linto)(microenable *board);

// -------------------------------------------------------------------------
// Readback Funktionen
//

extern int   (ME_FUNC_PTR InitReadback)( const char* BaseName,unsigned int        ChipType);
extern unsigned int  (ME_FUNC_PTR GetRegisterState)( fpga_design* rdb_design,const char* Name, unsigned int from,unsigned int to );
extern unsigned int  (ME_FUNC_PTR GetPinState)( fpga_design* rdb_design,const char* Name );


// -------------------------------------------------------------------
//
// Info Strukturen
//

extern int	(ME_FUNC_PTR fpga2id)(char *fpga);
extern int	(ME_FUNC_PTR package2id)(char *package);
extern int	(ME_FUNC_PTR pn2id)(char *pn);
extern int	(ME_FUNC_PTR id2fpga)(char *fpga, int id);
extern int	(ME_FUNC_PTR id2package)(char *package, int id);
extern int	(ME_FUNC_PTR id2pn)(char *pn, int id);
extern int	(ME_FUNC_PTR get_next_serial)(FILE *fserial);
extern void (ME_FUNC_PTR get_info)(FILE *fserial, short_info *info);
extern void	(ME_FUNC_PTR write_info)(short_info *info, eeprom *my_eeprom, int serial);
extern void (ME_FUNC_PTR write_eeprom_defaults)(eeprom *my_eeprom);
extern unsigned long (ME_FUNC_PTR nr_fpga)(unsigned long pn);
extern unsigned long (ME_FUNC_PTR nr_ram_blocks)(unsigned long pn);
extern unsigned long (ME_FUNC_PTR ram_type)(unsigned long pn);
extern unsigned long (ME_FUNC_PTR get_actual_date)();
extern int	 (ME_FUNC_PTR get_dd_info)(microenable *board,unsigned long *ptr);
extern void	 (ME_FUNC_PTR printf_dd_info)(unsigned long *ptr);
extern char* (ME_FUNC_PTR GetActVersion)();
extern char* (ME_FUNC_PTR FPGAtype2String)(int FpgaID);
extern int	 (ME_FUNC_PTR String2FPGAtype)(char *fpga);
extern int   (ME_FUNC_PTR id2fpga)(char *fpga, int id); 
extern int   (ME_FUNC_PTR fpga2id)(char *fpga);
extern int           (ME_FUNC_PTR DetermineFPGAvendor)(int FpgaID, char *VendorString);
extern int           (ME_FUNC_PTR DetermineFPGAfamily)(int FpgaID, char *FPGAfamilyString);
extern int           (ME_FUNC_PTR DetermineFPGAinfo)(int FpgaID, fpga_design *des);
extern unsigned long (ME_FUNC_PTR DetermineFPGAidcode)(int FpgaID);
extern void          (ME_FUNC_PTR EvaluateIdcode)(unsigned long idcode, Idcode_struct *info);
extern int	         (ME_FUNC_PTR TestFPGAbitstreamKompatibility)(int FpgaID1, int FpgaID2);
extern char*         (ME_FUNC_PTR FPGAtype2RBTstring)(int FpgaID);
extern int           (ME_FUNC_PTR RBTstring2FPGAtype)(char *string);
extern int           (ME_FUNC_PTR GetRamSize)(microenable *board);
extern int           (ME_FUNC_PTR GetFPGAtype)(microenable *board);
extern int           (ME_FUNC_PTR GetSerialNumber)(microenable *board);
extern int           (ME_FUNC_PTR GetBusNumber)(microenable *board);
extern int			(ME_FUNC_PTR GetBoardVersion)(microenable *board);
extern int			(ME_FUNC_PTR GetBoardRevision)(microenable *board);
extern int			(ME_FUNC_PTR GetBoardType)(microenable *board);

extern int			(ME_FUNC_PTR MeGetFpgaType)(microenable *board);
extern int			(ME_FUNC_PTR CancelAllIo)(microenable *board);

extern int			(ME_FUNC_PTR GetFPGAPackage)(microenable *board);
extern int			(ME_FUNC_PTR GetFPGASpeedgrade)(microenable *board);
extern int			(ME_FUNC_PTR GetNrOfFPGAs)(microenable *board);
extern int			(ME_FUNC_PTR GetRamSpeed)(microenable *board);
extern int			(ME_FUNC_PTR GetRamType)(microenable *board);
extern int			(ME_FUNC_PTR GetInterruptNr)(microenable *board);

extern int			(ME_FUNC_PTR check_dd_version)(microenable *board,int *build,int *version);

extern int			(ME_FUNC_PTR GetTemperatureFPGA)   (microenable *board );
extern int			(ME_FUNC_PTR IsPowerInterrupt)   (microenable *board );

///////////////////////////////////////////////////////////////////////////
// Register Schnittstelle

extern int				(ME_FUNC_PTR GetNrOfRegister)		(fpga_design* design);
extern char*			(ME_FUNC_PTR GetRegisterName)		(fpga_design* design,int nr);
extern int				(ME_FUNC_PTR GetRegisterDirection)	(fpga_design* design,int nr);
extern hap_register*	(ME_FUNC_PTR GetRegisterByNumber)	(fpga_design* design,int nr);
extern hap_register*	(ME_FUNC_PTR GetRegisterByName)	(fpga_design* design,char *name);
extern int				(ME_FUNC_PTR RegisterWrite)		(microenable *pBoard,hap_register *hreg,unsigned long value);
extern unsigned long	(ME_FUNC_PTR RegisterRead)			(microenable *pBoard,hap_register *hreg);
extern int				(ME_FUNC_PTR RegisterReadEx)		(microenable *pBoard,hap_register *hreg,unsigned long*);
extern hap_register*	(ME_FUNC_PTR GetActRegisterByNumber)	(microenable *pBoard,int nr);
extern hap_register*	(ME_FUNC_PTR GetActRegisterByName)	(microenable *pBoard,char *name);
extern int				(ME_FUNC_PTR GetRegisterBitRange)	(fpga_design* design,int nr);
extern unsigned long	(ME_FUNC_PTR GetRegisterCacheValue)	(microenable *board,fpga_design* design,int nr);
extern int				(ME_FUNC_PTR GetRegisterBitRangeByName)	(fpga_design* design,char *name);

extern int		(ME_FUNC_PTR SetUserLock)(microenable* board);
extern int		(ME_FUNC_PTR FreeUserLock)(microenable* board);
extern int		(ME_FUNC_PTR IsConfigured)(microenable* board);
extern char		*(ME_FUNC_PTR GetDesignName)(microenable* board);

extern int 		(ME_FUNC_PTR test_mem)(microenable* board,unsigned long* ptr,unsigned long len);
extern int		(ME_FUNC_PTR GetHWStatus)(microenable *board);

///////////////////////////////////////////////////////////////////////////
// UIQ Schnittstelle

extern int				(ME_FUNC_PTR UIQR_readBuffer)			(microenable* board,int nBufferNr,unsigned char* cpBuffer,int nMaxNr,int timeoutInMs);
extern int				(ME_FUNC_PTR UIQR_readBufferLong)		(microenable* board,int nBufferNr,unsigned long* cpBuffer,int nMaxNr,int timeoutInMs);
extern int				(ME_FUNC_PTR UIQR_writeBuffer)			(microenable* board,int nBufferNr,unsigned char* cpBuffer,int nMaxNr,int timeoutInMs);
extern int				(ME_FUNC_PTR UIQR_getNrOfWords)		(microenable* board,int nBufferNr);
extern int				(ME_FUNC_PTR UIQR_getNrOfLostWords)	(microenable* board,int nBufferNr);
extern unsigned long	(ME_FUNC_PTR UIQR_getStatus)			(microenable* board,int nBufferNr);
extern int				(ME_FUNC_PTR UIQR_getNrOfInterrupts)	(microenable* board,int nBufferNr);
extern int				(ME_FUNC_PTR UIQR_waitUntilInterrupt)	(microenable* board,int nBufferNr);
extern int				(ME_FUNC_PTR UIQR_clearBuffer)			(microenable* board,int nBufferNr);

///////////////////////////////////////////////////////////////////////////

extern unsigned char*	(ME_FUNC_PTR MMGetBufferByNumber)		(microenable* board,dma_mem *dm,int pic_nr);
extern unsigned long    (ME_FUNC_PTR MMGetBufferSize)			(microenable* board,dma_mem *dm);
extern int				(ME_FUNC_PTR MMGetNrOfSubbuffer)			(microenable* board,dma_mem *dm);

extern int				(ME_FUNC_PTR MMgetBufferNumber)		(microenable* board,dma_mem *dm,int pic_nr);
extern int				(ME_FUNC_PTR MMunlockBuffer)			(microenable* board,dma_mem *dm,int buffer_nr);
extern int				(ME_FUNC_PTR MMTestBuffer)				(microenable* board,int size);
extern int 				(ME_FUNC_PTR MMTestUserBuffer)			(microenable* board,int size,unsigned long address);
extern int 				(ME_FUNC_PTR MMFreeUserBuffer)			(microenable* board,int nr);

///////////////////////////////////////////////////////////////////////////

extern unsigned long	(ME_FUNC_PTR GetLicenseStringAsLong)(microenable *board,int id);
extern unsigned long	(ME_FUNC_PTR SetLicenseStringAsLong)(microenable *board,int id,unsigned long value);

extern int				(ME_FUNC_PTR getLicenseUserCode)(microenable *board);
extern int				(ME_FUNC_PTR getLicenseGroupMask)(microenable *board);

#ifdef QNX
extern void				(ME_FUNC_PTR setInterruptEvent)(microenable* board,void *event);
#endif

#define DLL_TYP						0x100
#define GET_FOR_MANUFACTOR_STRING	0x200
#define GETCAMERASTRING				0x300
#define SETDLLPARAM					0x400

#define GET_NR_OF_GENFUNCS			0x500
#define GET_GENFUNCNAME				0x600
#define GET_NR_OF_PORTS				0x700
#define GET_NR_OF_CAMS				0x750
#define GET_VERSION					0x800
#define GET_INSTALL_VERSION			0x850
#define GET_HAP_NAME				0x870
#define GET_BOARD_TYP				0x880

#define INSTALL_VERSION	 3
#define INSTALL_REVISION 1

#define GETLICENSEINFO				0x900

typedef int infofunc(int,int,int,void*,void**);
typedef infofunc* pinfofunc;

#define PN_MICROENABLE		0xa1
#define PN_MICROENABLEII	0xa2
#define PN_MICROENABLE3I	0xa3
#define PN_MICROENABLE3IXXL	0xa31

#define SEL_ACT_IMAGE		200
#define SEL_LAST_IMAGE		210
#define SEL_NEXT_IMAGE		220
#define SEL_NUMBER			230
#define SEL_NEW_IMAGE		240

#define CONTMODE 0x10
#define CONTHANDSHAKEMODE 0x20

#ifdef __cplusplus
}
#endif

#endif
